<?php

namespace Drupal\connectorg_extra_functions;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\user\Entity\User;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;

/**
 * HelperService service.
 *
 * @package Drupal\connectorg_extra_functions
 * @author Jose <jperez@weknowinc.com>
 */
class HelperService {

  /**
   * The logger channel factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Constructs a HelperService object.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger channel factory.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   */
  public function __construct(LoggerChannelFactoryInterface $logger, ClientInterface $http_client) {
    $this->logger = $logger->get('connectorg_extra_functions');
    $this->httpClient = $http_client;
  }

  /**
   * Exploded string.
   *
   * @param string $string
   *   The string.
   *
   * @return mixed|string
   *   Return exploded string.
   */
  private function explodedString(string $string) {
    $exploded = explode("@", $string);
    return $exploded[0];
  }

  /**
   * @param \Drupal\connectorg_employee_engagement\Entity\StarEntity $starEntity
   */
  private function formatParams(EntityInterface $starEntity): array {
    $employeeId = $starEntity->get("field_employee")->target_id;
    $userId = $starEntity->get("user_id")->target_id;
    $sentBy = User::load($userId);
    $sentByUser = $employeeUser = NULL;
    if ($sentBy) {
      $sentByUser = $this->explodedString($sentBy->getEmail());
    }
    $employee = User::load($employeeId);
    if ($employee) {
      $employeeUser = $this->explodedString($employee->getEmail());
    }
    return [
      "ID" => $starEntity->id(),
      "Employee" => $employeeUser,
      "Message" => trim(strip_tags($starEntity->get("field_message")->value)),
      "Sentby" => $sentByUser,
      "Created date" => date("Y-m-d", $starEntity->get("created")->value),
    ];
  }

  /**
   * @param \Drupal\connectorg_employee_engagement\Entity\StarEntity $starEntity
   */
  public function sendDataTo(EntityInterface $starEntity): void {
    try {
      $params = $this->formatParams($starEntity);
      $body = '
      {
    "fields": {
        "ID": "' . $params['ID'] . '",
        "Employee": "' . $params['Employee'] . '",
        "Message": "' . $params['Message'] . '",
        "Sentby": "' . $params['Sentby'] . '",
        "Created date": "' . $params['Created date'] . '"
    }
}
      ';
      $response = $this->httpClient->post(
        "https://api.airtable.com/v0/appoGH1eE2PCcmzIK/Stars",
        [
          "body" => $body,
          "headers" => [
            'Authorization' => 'Bearer keybLkErDBNGwFVn2',
            'Content-Type' => 'application/json',
            "accept" => "application/json",
            'Cache-Control' => 'no-cache, no-store, must-revalidate',
            'Pragma' => 'no-cache',
            'Expires' => '0',
          ],
        ]
      );
      if (
        $response &&
        $response->getStatusCode() === 200 &&
        $contents = $response->getBody()->getContents()
      ) {
        $this->logger->log("success", "Successful action: " . $contents);
      }
    }
    catch (\Exception | GuzzleException $exception) {
      $this->logger->error($exception->getTraceAsString());
    }
  }

}
