<?php

namespace Drupal\connectorg_extra_functions\Plugin\views\filter;

use Drupal\views\Annotation\ViewsFilter;
use Drupal\views\Plugin\views\filter\NumericFilter;

/**
 * Filters by given list of node title options.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("d8_connectorg_download_stars_filter")
 */
class DownloadStarCustomFilter extends NumericFilter {

  /**
   * {@inheritdoc}
   */
  public function query(): void {
    $beforeDayStart = strtotime(date("Y-m-d", time() - 86400) . "T00:00:00");
    $beforeDayEnd = strtotime(date("Y-m-d", time() - 86400) . "T23:59:59");
    $this->query->addWhere('AND', 'star_entity.created', $beforeDayStart, ">=");
    $this->query->addWhere('AND', 'star_entity.created', $beforeDayEnd, "<=");
  }

  /**
   * {@inheritdoc}
   */
  public function operators(): array {
    $operators = parent::operators();
    $operators += [
      'before date' => [
        'title' => $this->t('Is this day of the week'),
        'short' => $this->t('before date'),
        'method' => 'op_day_of_week',
        'min' => 0,
        'max' => 0,
        'values' => 0,
      ],
    ];
    return $operators;
  }

}